﻿Imports System.Collections
Imports System.Data.Entity
Imports NinjaDomain.Classes

Namespace NinjaDomain.DataModel
  Public Class DisconnectedRepository
	Public Sub New()
	  'this is for purposes only, to make it easier to share this demo. Is not guidance for production apps! :)
	  'this will only create and seed if db doesn't exist or ninjas is empty
	   DataHelpers.NewDbWithSeed()
	End Sub
	Public Function GetQueryableNinjasWithClan(ByVal query As String, ByVal page As Integer, ByVal pageSize As Integer) As List(Of Ninja)
	  Using context = New NinjaContext()
                context.Database.Log = Function(message)
                                           Debug.WriteLine(message)
                                       End Function
                Dim linqQuery = context.Ninjas.Include(Function(n) n.Clan)
		If Not String.IsNullOrEmpty(query) Then
		  linqQuery = linqQuery.Where(Function(n) n.Name.Contains(query))
		End If
		If page > 0 AndAlso pageSize > 0 Then
		  linqQuery = linqQuery.OrderBy(Function(n) n.Name).Skip(page - 1).Take(pageSize)
		End If

		Return linqQuery.ToList()
	  End Using
	End Function

	Public Function GetNinjasWithClan() As List(Of Ninja)
	  Using context = New NinjaContext()
		'return context.Ninjas.Include(n => n.Clan).ToList();
		Return context.Ninjas.AsNoTracking().Include(Function(n) n.Clan).ToList()
	  End Using
	End Function

	Public Function GetNinjaWithEquipment(ByVal id As Integer) As Ninja
	  Using context = New NinjaContext()
		Return context.Ninjas.AsNoTracking().Include(Function(n) n.EquipmentOwned).FirstOrDefault(Function(n) n.Id = id)
	  End Using
	End Function

	Public Function GetNinjaWithEquipmentAndClan(ByVal id As Integer) As Ninja
	  Using context = New NinjaContext()
		Return context.Ninjas.AsNoTracking().Include(Function(n) n.EquipmentOwned).Include(Function(n) n.Clan).FirstOrDefault(Function(n) n.Id = id)
	  End Using
	End Function

	Public Function GetClanList() As IEnumerable
	  Using context = New NinjaContext()
		Return context.Clans.AsNoTracking().OrderBy(Function(c) c.ClanName).Select(Function(c) New With {Key c.Id, Key c.ClanName}).ToList()
	  End Using
	End Function

	Public Function GetNinjaById(ByVal id As Integer) As Ninja
	  Using context = New NinjaContext()
		Return context.Ninjas.Find(id)
		' return context.Ninjas.AsNoTracking().SingleOrDefault(n => n.Id == id);
	  End Using
	End Function

	Public Sub SaveUpdatedNinja(ByVal ninja As Ninja)
	  Using context = New NinjaContext()
		context.Entry(ninja).State = EntityState.Modified
		context.SaveChanges()
	  End Using
	End Sub

	Public Sub SaveNewNinja(ByVal ninja As Ninja)
	  Using context = New NinjaContext()
		context.Ninjas.Add(ninja)
		context.SaveChanges()
	  End Using
	End Sub

	Public Sub DeleteNinja(ByVal ninjaId As Integer)
	  Using context = New NinjaContext()
		Dim ninja = context.Ninjas.Find(ninjaId)
		context.Entry(ninja).State = EntityState.Deleted
		context.SaveChanges()
	  End Using
	End Sub

	Public Sub SaveNewEquipment(ByVal equipment As NinjaEquipment, ByVal ninjaId As Integer)
	  'paying the price of not having a foreign key here. 
	  'reason #857 why I prefer foreign keys!
	  Using context = New NinjaContext()
		Dim ninja = context.Ninjas.Find(ninjaId)
		ninja.EquipmentOwned.Add(equipment)

		context.SaveChanges()
	  End Using
	End Sub

	Public Sub SaveUpdatedEquipment(ByVal equipment As NinjaEquipment, ByVal ninjaId As Integer)
	  'paying the price of not having a foreign key here. 
	  'reason #858 why I prefer foreign keys!
	  Using context = New NinjaContext()
		Dim equipmentWithNinjaFromDatabase = context.Equipment.Include(Function(n) n.Ninja).FirstOrDefault(Function(e) e.Id = equipment.Id)

		context.Entry(equipmentWithNinjaFromDatabase).CurrentValues.SetValues(equipment)

		context.SaveChanges()
	  End Using
	End Sub


	Public Function GetEquipmentById(ByVal id As Integer) As NinjaEquipment
	  Using context = New NinjaContext()
		Return context.Equipment.Find(id)
	  End Using
	End Function
  End Class
End Namespace